<?php

namespace Evoluted\Sniffs\Classes;

use PHP_CodeSniffer\Sniffs\Sniff;
use PHP_CodeSniffer\Files\File;

class ConstantsBeforePropertiesSniff implements Sniff
{
    public function process(File $phpcsFile, $stackPtr)
    {
        $tokens = $phpcsFile->getTokens();
        $constant = $stackPtr;

        // this is the end position of the class (if any)
        $end = null;
        if (isset($tokens[$stackPtr]['scope_closer'])) {
            $end = $tokens[$stackPtr]['scope_closer'];
        }

        while ($constant) {
            $constant = $phpcsFile->findNext(
                T_CONST,
                $constant + 1,
                $end
            );

            if ($constant === false) {
                break;
            }

            $variable = $phpcsFile->findPrevious(T_VARIABLE, $constant - 1, $stackPtr);

            if ($variable !== false && $variable < $constant) {
                $error = 'Constants must be defined before all class properties';
                $phpcsFile->addError($error, $constant, 'Invalid');
            }
        }
    }

    public function register(): array
    {
        return [ T_CLASS, T_INTERFACE ];
    }
}