# Evoluted PHP coding standard

Because we all have standards.

## Quick start

* Grab the latest version of [PHP-CS](https://github.com/squizlabs/PHP_CodeSniffer)
* Add this repo to your project: `composer require --dev evoluted/coding-standard:version` - always lock to a version
* Ensure that PHP-CS can find the standard: `phpcs --config-set installed_paths vendor/evoluted/coding-standard`, then `phpcs -i` should list "Evoluted"
* Run `phpcs --standard=Evoluted app/`
* **Optional**: copy the [`phpcs-sample.xml`](phpcs-sample.xml) file to your project as `phpcs.xml` to be able to run without the `--standard` option e.g. `phpcs app/`

## About

[PSR-2 coding standards](https://github.com/php-fig/fig-standards/blob/master/accepted/PSR-2-coding-style-guide.md) are a great starting point for standards but are, by their nature, very broad.

**[Very broad](examples/BadPsr2.php)**

The Evoluted PHP coding standard builds on PSR-2 with a series of standards aimed at making all of the code we produce similarly styled. This means that when it comes to code reviews or bug fixes you can focus on what the code does, and not about how the code looks.

## Details

The Evoluted coding standards are:

* Built on top of PSR-2
* Designed to reduce the dissonance when switching projects
* For new and modern projects only - Cake has its own *particular* style
* Built upon consensus and best practice
* Runnable from the PHP-CS command line tool
* Replete with examples
* Versioned
* Always evolving

## Process

These standards will start out with a few code sniffs, with the aim of adding new ones as they are come across and in a maintainable manner.

New sniffs are added through a process of recommendation, discussion, voting, and finally implementation.

Each new set of sniffs added to the standard will be semver tagged.

## Current standards

* [PSR-2](https://github.com/php-fig/fig-standards/blob/master/accepted/PSR-2-coding-style-guide.md)
* [Always use short array syntax](standards/suggested/ShortArraySyntax.md)
* [Methods in classes should be ordered: public, protected, private](standards/suggested/ClassMethodOrdering.md)
* [Properties in classes should come before methods](standards/suggested/ClassPropertyOrdering.md)
* [Always have a blank line before a return statement](standards/suggested/BlankLineBeforeReturn.md)

## Upcoming standards

## Suggested standards

Want to suggest a standard? Create a markdown document that details your suggestion and commit it straight into the `standards/suggested` directory. You should also create a "good" implementation of the suggestion as well as "bad" one and commit into the `examples/suggested` directory. Ideally you should also have a working sniff in mind to implement your suggestion, this will eventually be added to the `Evoluted/ruleset.xml` file.
