# Standard: Class constant ordering

Constants should be defined before properties in a class. The order would be Constants, Properties, Methods.

## Reasoning

Public constants are often the most used part of a class so should be defined first, while scoped constants (protected, private, PHP 7.1+) are important for extending classes and it makes sense to keep all constants together for private constants.

## Good

```php
class MyClass
{
    const FOO = 'bar';

    public $one = '1';
    protected $_two = '2';
    private $__three = '3';
    
    public function one()
    {
    }
}
```

## Bad

```php
class MyClass
{
    public $one = '1';
 
    public function one()
    {
    }

    protected $_two = '2';
    private $__three = '3';

    const FOO = 'bar';
}
```
